import 'dart:io';

import 'package:args/args.dart';
import 'package:path/path.dart';

import 'src/server.dart';

Future main(List<String> args) async {
  final parser = ArgParser()
    ..addFlag(
      'help',
      abbr: 'h',
      negatable: false,
      defaultsTo: false,
    )
    ..addOption(
      'port',
      help: 'The port to listen on.',
      defaultsTo: '2658',
    )
    ..addOption(
      'path',
      help: 'The path to the Musicus database.',
    )
    ..addFlag(
      'tls',
      defaultsTo: false,
      help: 'Whether to use a TLS encrypted connection.',
    )
    ..addOption(
      'certificate',
      help: 'A path to a TLS certificate file (full chain).',
    )
    ..addOption(
      'key',
      help: 'A path to a TLS private key file.',
    );

  final options = parser.parse(args);

  if (options['help'] == true) {
    print(parser.usage);
    return 0;
  }

  final port = int.parse(options['port']);
  final path = options['path'] ?? current;

  HttpServer http;

  if (options['tls']) {
    final certificate = options['certificate'];
    final key = options['key'];

    final securityContext = SecurityContext()
      ..useCertificateChain(certificate)
      ..usePrivateKey(key);

    http = await HttpServer.bindSecure(
        InternetAddress.anyIPv4, port, securityContext);
  } else {
    http = await HttpServer.bind('localhost', port);
  }

  final server = Server(path);

  await for (final request in http) {
    try {
      await server.respond(request);
    } catch (exception) {
      print('${DateTime.now()} – $exception');
    }
  }
}
