import 'dart:io';
import 'package:path/path.dart';

import 'package:musicus_shared/musicus_shared.dart';

class ServerLibrary extends Library {
  final String path;
  String get erasPath => join(path, 'eras');
  String get musiciansPath => join(path, 'musicians');
  String get genresPath => join(path, 'genres');
  String get keysPath => join(path, 'keys');
  String get worksPath => join(path, 'works');

  ServerLibrary(this.path) {
    Directory(erasPath).createSync();
    Directory(musiciansPath).createSync();
    Directory(genresPath).createSync();
    Directory(keysPath).createSync();
    Directory(worksPath).createSync();
  }

  @override
  Future<int> addEra(Era era) async {
    final int id = Library.generateId();
    final file = File(join(erasPath, id.toString()));
    await file.writeAsString(era.toString());

    print('${DateTime.now()} – Added era $id.');
    return id;
  }

  @override
  Future<int> addMusician(Musician musician) async {
    final int id = Library.generateId();
    final file = File(join(musiciansPath, id.toString()));
    await file.writeAsString(musician.toString());

    print('${DateTime.now()} – Added musician $id.');
    return id;
  }

  @override
  Future<int> addGenre(Genre genre) async {
    final int id = Library.generateId();
    final file = File(join(genresPath, id.toString()));
    await file.writeAsString(genre.toString());

    print('${DateTime.now()} – Added genre $id.');
    return id;
  }

  @override
  Future<int> addKey(Key key) async {
    final int id = Library.generateId();
    final file = File(join(keysPath, id.toString()));
    await file.writeAsString(key.toString());

    print('${DateTime.now()} – Added key $id.');
    return id;
  }

  @override
  Future<int> addWork(Work work) async {
    final int id = Library.generateId();
    final file = File(join(worksPath, id.toString()));
    await file.writeAsString(work.toString());

    print('${DateTime.now()} – Added work $id.');
    return id;
  }

  @override
  Future<void> updateEra(int id, Era era) async {
    final file = File(join(erasPath, id.toString()));
    await file.writeAsString(era.toString());
    print('${DateTime.now()} – Updated era $id.');
  }

  @override
  Future<void> updateMusician(int id, Musician musician) async {
    final file = File(join(musiciansPath, id.toString()));
    await file.writeAsString(musician.toString());
    print('${DateTime.now()} – Updated musician $id.');
  }

  @override
  Future<void> updateGenre(int id, Genre genre) async {
    final file = File(join(genresPath, id.toString()));
    await file.writeAsString(genre.toString());
    print('${DateTime.now()} – Updated genre $id.');
  }

  @override
  Future<void> updateKey(int id, Key key) async {
    final file = File(join(keysPath, id.toString()));
    await file.writeAsString(key.toString());
    print('${DateTime.now()} – Updated key $id.');
  }

  @override
  Future<void> updateWork(int id, Work work) async {
    final file = File(join(worksPath, id.toString()));
    await file.writeAsString(work.toString());
    print('${DateTime.now()} – Updated work $id.');
  }

  @override
  Future<void> deleteEra(int id) async {
    final file = File(join(erasPath, id.toString()));
    await file.delete();
    print('${DateTime.now()} – Deleted era $id.');
  }

  @override
  Future<void> deleteMusician(int id) async {
    final file = File(join(musiciansPath, id.toString()));
    await file.delete();
    print('${DateTime.now()} – Deleted musician $id.');
  }

  @override
  Future<void> deleteGenre(int id) async {
    final file = File(join(genresPath, id.toString()));
    await file.delete();
    print('${DateTime.now()} – Deleted genre $id.');
  }

  @override
  Future<void> deleteKey(int id) async {
    final file = File(join(keysPath, id.toString()));
    await file.delete();
    print('${DateTime.now()} – Deleted key $id.');
  }

  @override
  Future<void> deleteWork(int id) async {
    final file = File(join(worksPath, id.toString()));
    await file.delete();
    print('${DateTime.now()} – Deleted work $id.');
  }

  @override
  Future<Era> getEra(int id) async {
    final file = File(join(erasPath, id.toString()));
    final content = await file.readAsString();
    return Era.fromString(content);
  }

  @override
  Future<Musician> getMusician(int id) async {
    final file = File(join(musiciansPath, id.toString()));
    final content = await file.readAsString();
    return Musician.fromString(content);
  }

  @override
  Future<Genre> getGenre(int id) async {
    final file = File(join(genresPath, id.toString()));
    final content = await file.readAsString();
    return Genre.fromString(content);
  }

  @override
  Future<Key> getKey(int id) async {
    final file = File(join(keysPath, id.toString()));
    final content = await file.readAsString();
    return Key.fromString(content);
  }

  @override
  Future<Work> getWork(int id) async {
    final file = File(join(worksPath, id.toString()));
    final content = await file.readAsString();
    return Work.fromString(content);
  }

  @override
  Future<Iterable<int>> getEras() async {
    final Set<int> result = {};

    final directory = Directory(erasPath);
    await for (final entity in directory.list()) {
      if (entity is File) {
        final id = int.tryParse(basename(entity.path));
        if (id != null) {
          result.add(id);
        }
      }
    }

    return result;
  }

  @override
  Future<Iterable<int>> getMusicians() async {
    final Set<int> result = {};

    final directory = Directory(musiciansPath);
    await for (final entity in directory.list()) {
      if (entity is File) {
        final id = int.tryParse(basename(entity.path));
        if (id != null) {
          result.add(id);
        }
      }
    }

    return result;
  }

  @override
  Future<Iterable<int>> getGenres() async {
    final Set<int> result = {};

    final directory = Directory(genresPath);
    await for (final entity in directory.list()) {
      if (entity is File) {
        final id = int.tryParse(basename(entity.path));
        if (id != null) {
          result.add(id);
        }
      }
    }

    return result;
  }

  @override
  Future<Iterable<int>> getKeys() async {
    final Set<int> result = {};

    final directory = Directory(keysPath);
    await for (final entity in directory.list()) {
      if (entity is File) {
        final id = int.tryParse(basename(entity.path));
        if (id != null) {
          result.add(id);
        }
      }
    }

    return result;
  }

  @override
  Future<Iterable<int>> getWorks() async {
    final Set<int> result = {};

    final directory = Directory(worksPath);
    await for (final entity in directory.list()) {
      if (entity is File) {
        final id = int.tryParse(basename(entity.path));
        if (id != null) {
          result.add(id);
        }
      }
    }

    return result;
  }
}
